@@ -7,11 +7,11 @@ BEGIN {
 
 =head1 NAME
 
-Baldrick
+Baldrick - Baldrick Application Framework
 
 =head1 SYNOPSIS
 
-  use Baldrick;
+  use Baldrick::Baldrick;
 
 =head1 DESCRIPTION
 
@@ -35,7 +35,6 @@ it and/or modify it under the same terms as Perl itself.
 The full text of the license can be found in the
 LICENSE file included with this module.
 
-
 =head1 SEE ALSO
 
 Baldrick::Stub
